#
# Copyright John Reid 2006, 2007, 2008, 2012
#


import pkg_resources

__doc__ = pkg_resources.resource_string(__name__, "README")
__license__ = pkg_resources.resource_string(__name__, "LICENSE")
__release__, __svn_revision__ = pkg_resources.resource_string(__name__, "VERSION").strip().split('-')
__major_version__, __minor_version__, __release_version__ = map(int, __release__.split('.'))
__version__ = '%d.%d' % (__major_version__, __minor_version__)

def version_string():
    """Return the release and svn revision as a string."""
    return '%s %s' % (__release__, __svn_revision__)



from bunch import *
from bidirectional_map import *
from const import *
from dicts import *
from enum import *
from equivalence import *
from groupby import *
from lru_cache import *
from named_tuple import *
from pre_post_conditions import *
from process_priority import *
from reverse_dict import *





#
# Copyright John Reid 2008,2009
#


"""
Utility functions for simple logging operations.
"""


import logging, sys, functools


def basic_config(level=logging.INFO):
    """
    Set up simple logging to stderr.
    """
    logging.basicConfig(level=level)


def add_logging_to_file(filename, mode='w'):
    """
    Add logging to named file.
    """
    file_log = logging.FileHandler(filename, mode)
    logging.getLogger('').addHandler(file_log)


def log_to_file_and_stderr(filename, level=logging.INFO, mode='w'):
    """
    Set up simple logging to named file.
    """
    if len(logging.getLogger().handlers) < 2:
        basic_config(level)
        add_logging_to_file(filename, mode)
        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        for handler in logging.getLogger().handlers:
            handler.setFormatter(formatter)


def log_exception(level=logging.ERROR, logger=None):
    """
    Log the current exception.
    """
    if None == logger:
        logger = logging.getLogger('')
    logger.log(level, '%s: %s', sys.exc_info()[0].__name__, sys.exc_info()[1])



def log_exception_traceback(level=logging.ERROR, logger=None):
    """
    Log the traceback for the current exception.
    """
    import traceback, cStringIO
    tb = cStringIO.StringIO()
    traceback.print_tb(sys.exc_info()[2], file=tb)
    if None == logger:
        logger = logging.getLogger('')
    #logger.log(level, 'Caught exception whilst in function: %s()', fn.__name__)
    logger.log(level, 'Traceback:\n%s', tb.getvalue())


def create_log_exceptions_decorator(level=logging.ERROR, log_traceback=True, logger=None):
    """
    @return: Decorator to log exceptions raised inside a function.
    """
    if None == logger:
        logger = logging.getLogger('')

    def decorator(fn):
        @functools.wraps(fn)
        def wrapped(*args, **kwds):
            try:
                return fn(*args, **kwds)
            except:
                if log_traceback:
                    log_exception_traceback(level, logger)
                log_exception(level, logger)
                raise
        return wrapped

    return decorator


if '__main__' == __name__:
    logging.basicConfig(level=logging.INFO)
    
    class A(object):
        @create_log_exceptions_decorator()
        def method(self):
            raise RuntimeError('error')

    @create_log_exceptions_decorator()
    def exception_raiser():
        "Test exception function."
        raise RuntimeError('Error occurred')

    print 'Fn name: %s' % exception_raiser.__name__
    print 'Fn docstring: %s' % exception_raiser.__doc__
    try:
        exception_raiser()
    except:
        logging.warning('exception_raiser() raised exception')
    try:
        A().method()
    except:
        logging.warning('A().method() raised exception')



"""
From: http://code.activestate.com/recipes/52308-the-simple-but-handy-collector-of-a-bunch-of-named/
(released under PSF license).
"""

class Bunch(object):
    """
    Often we want to just collect a bunch of stuff together, naming each item of the bunch; 
    a dictionary's OK for that, but a small do-nothing class is even handier, and prettier to use.
    """
    
    def __init__(self, **kwds):
        self.__dict__.update( kwds )

    def __str__(self):
        return 'Bunch:\n\t%s' % '\n\t'.join(
                [
                        '%s:\n%s' % (k,v) for k, v in self.__dict__.iteritems()
                ]
        )



"""

From:
http://code.activestate.com/recipes/473899-progress-meter/


Here is a silly example of its usage:

import progress
import time
import random

total = 1000
p = progress.ProgressMeter(total=total)

while total > 0:
    cnt = random.randint(1, 25)
    p.update(cnt)
    total -= cnt
    time.sleep(random.random())


Here is an example of its output:

[------------------------->                                   ] 41%  821.2/sec


2006-02-20 Denis Barmenkov: ANSI codes replaced by Backspace (0x08) characters
"""
import time, sys, math

class ProgressMeter(object):
    #ESC = chr(27)
    def __init__(self, **kw):
        # What time do we start tracking our progress from?
        self.timestamp = kw.get('timestamp', time.time())
        # What kind of unit are we tracking?
        self.unit = str(kw.get('unit', ''))
        # Number of units to process
        self.total = int(kw.get('total', 100))
        # Number of units already processed
        self.count = int(kw.get('count', 0))
        # Refresh rate in seconds
        self.rate_refresh = float(kw.get('rate_refresh', .5))
        # Number of ticks in meter
        self.meter_ticks = int(kw.get('ticks', 60))
        self.meter_division = float(self.total) / self.meter_ticks
        self.meter_value = int(self.count / self.meter_division)
        self.last_update = None
        self.rate_history_idx = 0
        self.rate_history_len = 10
        self.rate_history = [None] * self.rate_history_len
        self.rate_current = 0.0
        self.last_refresh = 0
        self.prev_meter_len = 0

    def update(self, count, **kw):
        now = time.time()
        # Caclulate rate of progress
        rate = 0.0
        # Add count to Total
        self.count += count
        self.count = min(self.count, self.total)
        if self.last_update:
            delta = now - float(self.last_update)
            if delta:
                rate = count / delta
            else:
                rate = count
            self.rate_history[self.rate_history_idx] = rate
            self.rate_history_idx += 1
            self.rate_history_idx %= self.rate_history_len
            cnt = 0
            total = 0.0
            # Average rate history
            for rate in self.rate_history:
                if rate == None:
                    continue
                cnt += 1
                total += rate
            rate = total / cnt
        self.rate_current = rate
        self.last_update = now
        # Device Total by meter division
        value = int(self.count / self.meter_division)
        if value > self.meter_value:
            self.meter_value = value
        if self.last_refresh:
            if (now - self.last_refresh) > self.rate_refresh or \
                (self.count >= self.total):
                self.refresh()
        else:
            self.refresh()

    def get_meter(self, **kw):
        bar = '-' * self.meter_value
        pad = ' ' * (self.meter_ticks - self.meter_value)
        perc = (float(self.count) / self.total) * 100
        return '[%s>%s] %d%%  %.1f/sec' % (bar, pad, perc, self.rate_current)

    def refresh(self, **kw):
        # Clear line and return cursor to start-of-line
        sys.stdout.write(' ' * self.prev_meter_len + '\x08' * self.prev_meter_len)
        # Get meter text
        meter_text = self.get_meter(**kw)
        # Write meter and return cursor to start-of-line
        sys.stdout.write(meter_text + '\x08'*len(meter_text))
        self.prev_meter_len = len(meter_text)

        # Are we finished?
        if self.count >= self.total:
            sys.stdout.write('\n')
        sys.stdout.flush()
        # Timestamp
        self.last_refresh = time.time()



def equivalence_partition( iterable, relation ):
    """
    Partitions a set of objects into equivalence classes

    :param iterable:
        collection of objects to be partitioned
    :param relation:
        equivalence relation. I.e. relation(o1,o2) evaluates to True
        if and only if o1 and o2 are equivalent

    :returns: (classes, partitions)
        classes is a sequence of sets. Each one is an equivalence class.
        partitions is a dictionary mapping objects to equivalence classes.
    """
    classes = []
    partitions = { }
    for o in iterable: # for each object
        # find the class it is in
        for c in classes:
            if relation( iter(c).next(), o ): # is it equivalent to this class?
                c.add( o )
                partitions[o] = c
                break
        else: # it is in a new class
            classes.append( set( [ o ] ) )
            partitions[o] = classes[-1]
    return classes, partitions

def equivalence_enumeration( iterable, relation ):
    """Partitions a set of objects into equivalence classes

    Same as equivalence_partition() but also numbers the classes.

    :param iterable: collection of objects to be partitioned
    :param relation: equivalence relation. I.e. relation(o1,o2) evaluates to True
        if and only if o1 and o2 are equivalent

    :returns: (classes, partitions, ids)
        classes is a sequence of sets. Each one is an equivalence class
        partitions is a dictionary mapping objects to equivalence classes
        ids is a dictionary mapping objects to the indices of their equivalence classes
    """
    classes, partitions = equivalence_partition( iterable, relation )
    ids = { }
    for i, c in enumerate( classes ):
        for o in c:
            ids[o] = i
    return classes, partitions, ids

def check_equivalence_partition( classes, partitions, relation ):
    """Checks that a partition is consistent under the relationship"""
    for o, c in partitions.iteritems():
        for _c in classes:
            assert (o in _c) ^ (not _c is c)
    for c1 in classes:
        for o1 in c1:
            for c2 in classes:
                for o2 in c2:
                    assert (c1 is c2) ^ (not relation( o1, o2 ))

def test_equivalence_partition():
    relation = lambda x,y: (x-y) % 4 == 0
    classes, partitions = equivalence_partition(
            xrange( -3, 5 ),
            relation
    )
    check_equivalence_partition( classes, partitions, relation )
    for c in classes: print c
    for o, c in partitions.iteritems(): print o, ':', c



#
# Copyright John Reid 2010
#


"""
Code to implement output contexts.
"""

import os, glob, logging
from cookbook.cache_decorator import PickledMemoize



def ensure_dir_exists(dir):
    "Make sure the given directory exists."
    if not os.path.exists(dir):
        os.makedirs(dir)



class OutputContext(object):
    """
    An output context. Uses the Borg design pattern:
    http://code.activestate.com/recipes/66531-singleton-we-dont-need-no-stinkin-singleton-the-bo/
    """
    
    __shared_state = {}
    
    def __init__(self):
        "Construct."
        self.__dict__ = self.__shared_state

    
    def setUp(self, root_dir, tag=None):
        "Set up the output context, i.e. tell it where any output should go."
        self.root_dir = root_dir
        if None == tag: # if no tag supplied, work out which is next.
            existing_output_dirs = glob.glob(os.path.join(self.root_dir, '[0-9][0-9][0-9][0-9]'))
            if len(existing_output_dirs):
                existing_tags = map(os.path.basename, existing_output_dirs)
                existing_tags.sort()
                tag = '%04d' % (int(existing_tags[-1]) + 1)
            else:
                tag = '0000'
        self.tag = tag
        self.output_dir = os.path.join(self.root_dir, self.tag)
        logging.info('Setting up output context to send output to %s', self.output_dir)
        self.cache_dir = os.path.join(self.output_dir, 'cache')
        ensure_dir_exists(self.cache_dir)


    def _check_is_setUp(self):
        try:
            self.cache_dir
        except AttributeError:
            logging.warning('output_context.setUp() never called, using default settings.')
            self.setUp('output')
    
    
    def cache_filename(self, object_name):
        "@return: A callable that returns a filename suitable for caching an object."
        def filename():
            self._check_is_setUp()
            return os.path.join(self.cache_dir, '%s.pickle' % object_name)
        return filename
    
    
    def caching_decorator(self, name, dump_on_update=False):
        "@return: A decorator that caches the results of methods."
        def decorator(fn):
            return PickledMemoize(fn, self.cache_filename(name), dump_on_update=dump_on_update)
        return decorator


output_context = OutputContext()
"An object we can use to access the output context."


if '__main__' == __name__:
    logging.basicConfig(level=logging.DEBUG)
    
    logging.debug('Defining cached function.')
    @output_context.caching_decorator('square', dump_on_update=True)   
    def square(x):
        return x**2
    
    logging.debug('Setting up output context.')
    output_context.setUp('output')
        
    logging.debug('Calling cached function.')
    print square(9)
    print square(9)
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright © 2012 Josh VanderLinden
#

"""
Calculates the total number of downloads that a particular PyPI package has
received across all versions tracked by PyPI

From http://www.codekoala.com/blog/2010/pypi-download-stats/

Copyright © 2012 Josh VanderLinden
"""

from datetime import datetime
import locale
import sys
import xmlrpclib

locale.setlocale(locale.LC_ALL, '')

class PyPIDownloadAggregator(object):

    def __init__(self, package_name, include_hidden=True):
        self.package_name = package_name
        self.include_hidden = include_hidden
        self.proxy = xmlrpclib.Server('http://pypi.python.org/pypi')
        self._downloads = {}

        self.first_upload = None
        self.first_upload_rel = None
        self.last_upload = None
        self.last_upload_rel = None

    @property
    def releases(self):
        """Retrieves the release number for each uploaded release"""

        result = self.proxy.package_releases(self.package_name, self.include_hidden)

        if len(result) == 0:
            # no matching package--search for possibles, and limit to 15 results
            results = self.proxy.search({
                'name': self.package_name,
                'description': self.package_name
            }, 'or')[:15]

            # make sure we only get unique package names
            matches = []
            for match in results:
                name = match['name']
                if name not in matches:
                    matches.append(name)

            # if only one package was found, return it
            if len(matches) == 1:
                self.package_name = matches[0]
                return self.releases

            error = """No such package found: %s

Possible matches include:
%s
""" % (self.package_name, '\n'.join('\t- %s' % n for n in matches))

            sys.exit(error)

        return result

    @property
    def downloads(self, force=False):
        """Calculate the total number of downloads for the package"""

        if len(self._downloads) == 0 or force:
            for release in self.releases:
                urls = self.proxy.release_urls(self.package_name, release)
                self._downloads[release] = 0
                for url in urls:
                    # upload times
                    uptime = datetime.strptime(url['upload_time'].value, "%Y%m%dT%H:%M:%S")
                    if self.first_upload is None or uptime < self.first_upload:
                        self.first_upload = uptime
                        self.first_upload_rel = release

                    if self.last_upload is None or uptime > self.last_upload:
                        self.last_upload = uptime
                        self.last_upload_rel = release

                    self._downloads[release] += url['downloads']

        return self._downloads

    def total(self):
        return sum(self.downloads.values())

    def average(self):
        return self.total() / len(self.downloads)

    def max(self):
        return max(self.downloads.values())

    def min(self):
        return min(self.downloads.values())

    def stats(self):
        """Prints a nicely formatted list of statistics about the package"""

        self.downloads # explicitly call, so we have first/last upload data
        fmt = locale.nl_langinfo(locale.D_T_FMT)
        sep = lambda s: locale.format('%d', s, 3)
        val = lambda dt: dt and dt.strftime(fmt) or '--'

        params = (
            self.package_name,
            val(self.first_upload),
            self.first_upload_rel,
            val(self.last_upload),
            self.last_upload_rel,
            sep(len(self.releases)),
            sep(self.max()),
            sep(self.min()),
            sep(self.average()),
            sep(self.total()),
        )

        print """PyPI Package statistics for: %s

    First Upload: %40s (%s)
    Last Upload:  %40s (%s)
    Number of releases: %34s
    Most downloads:    %35s
    Fewest downloads:  %35s
    Average downloads: %35s
    Total downloads:   %35s
""" % params

def main():
    if len(sys.argv) < 2:
        sys.exit('Please specify at least one package name')

    for pkg in sys.argv[1:]:
        PyPIDownloadAggregator(pkg).stats()

if __name__ == '__main__':
    main()


#
# Copyright John Reid 2010, 2011
#

"""
Code to simplify logging and options in scripts.
"""

import traceback, logging, os, sys


def prepend_to_environment_variable(var_name, value):
    "Prepend a value to colon separated list in the named environment variable."
    # prepend this directory to front of LD_LIBRARY_PATH environment variable
    existing_value = os.environ.get(var_name, '')
    if existing_value:
        new_value = '%s:%s' % (value, existing_value)
    else:
        new_value = value
    os.environ[var_name] = new_value
    logging.debug('Changed environment variable %s to %s', var_name, os.environ[var_name])


def log_exception(exc_type, exc_obj, exc_tb):
    "Log an exception. Used as replacement for sys.excepthook."
    exc_info = traceback.format_exception(exc_type, exc_obj, exc_tb)
    for l in exc_info:
        logging.error(l.strip())


def setup_logging(file=None, level=logging.INFO, log_command_line=True):
    "Set up logging."
    format = "%(asctime)s:%(levelname)s: %(message)s"
    logging.basicConfig(level=level, format=format)
    formatter = logging.Formatter(format)
    if file:
        file_handler = logging.FileHandler('%s.log' % os.path.splitext(os.path.basename(file))[0])
        file_handler.setFormatter(formatter)
        file_handler.setLevel(level)
        logging.getLogger('').addHandler(file_handler)
    if log_command_line:
        logging.info('Python: %s', sys.version)
        logging.info('Command line: %s', ' '.join(sys.argv))
    sys.excepthook = log_exception # make sure we log exceptions. This doesn't work if running under ipython.


def log_options(option_parser, options):
    "Log the values of the options."
    logging.info('Options:')
    _log_options(option_parser.option_list, options)
    for option_group in option_parser.option_groups:
        _log_option_group(option_group, options)


def _log_options(option_list, options):
    for option in option_list:
        if option.dest:
            logging.info('%32s: %-32s * %s', option.dest, str(getattr(options, option.dest)), option.help)


def _log_option_group(option_group, options):
    logging.info(option_group.title)
    logging.info(option_group.get_description())
    _log_options(option_group.option_list, options)


def _print_var(name):
    print '%s=%s' % (name, os.environ[name])
    print '%s:\n\t' % name, os.environ[name].replace(':', '\n\t')


def show_environment():
    "Print some details about the environment python is running in."
    print 'PYTHON:\n', sys.version
    print 'PROCESS:'
    os.system('ps -ef|grep %d|grep -v "grep %d"' % (os.getpid(), os.getpid()))
    print 'CWD:\n\t', os.getcwd()
    _print_var('LD_LIBRARY_PATH')
    _print_var('PYTHONPATH')
    print 'Args:\n\t', '\n\t'.join(sys.argv)



#
# Copyright John Reid 2008
#



class BidirectionalMap(object):
    """
    A bidirectional map that maintains 2 dicts, one to map each way between elements of type 1 and type 2.

    The map is built by adding pairs of elements that are related.
    """

    def __init__(self):
        from cookbook import DictOfSets

        self.map1 = DictOfSets()
        "Maps from keys of type 1 to values of type 2."

        self.map2 = DictOfSets()
        "Maps from keys of type 2 to values of type 1."


    def add(self, v1, v2):
        """
        Add a relationship between v1 of type 1 and v2 of type 2.
        """
        self.map1[v1].add(v2)
        self.map2[v2].add(v1)


    def remove(self, v1, v2):
        """
        Removed a relationship between v1 of type 1 and v2 of type 2.

        Will throw key error if relationship is not in map.
        """
        self.map1[v1].remove(v2)
        self.map2[v2].remove(v1)


    def discard(self, v1, v2):
        """
        Removed a relationship between v1 of type 1 and v2 of type 2.

        Will do nothing if relationship is not in map.
        """
        self.map1[v1].discard(v2)
        self.map2[v2].discard(v1)



#
# Copyright John Reid 2010
#

"""
Code to run a function as a task. Useful to make sequential code parallel.

Uses pickling to pass arguments and get result back.
"""


import sys


class CalledProcessError(Exception):
    """This exception is raised when a process run by check_call() or
    check_output() returns a non-zero exit status.
    The exit status will be stored in the returncode attribute;
    check_output() will also store the output in the output attribute.
    """
    def __init__(self, returncode, cmd, output=None):
        self.returncode = returncode
        self.cmd = cmd
        self.output = output
    def __str__(self):
        return "Command '%s' returned non-zero exit status %d. Output was:\n%s" % (self.cmd, self.returncode, self.output)


def check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

    If the exit code was non-zero it raises a CalledProcessError.  The
    CalledProcessError object will have the return code in the returncode
    attribute and output in the output attribute.

    The arguments are the same as for the Popen constructor.  Example:

    >>> check_output(["ls", "-l", "/dev/null"])
    'crw-rw-rw- 1 root root 1, 3 Oct 18  2007 /dev/null\n'

    The stdout argument is not allowed as it is used internally.
    To capture standard error in the result, use stderr=STDOUT.

    >>> check_output(["/bin/sh", "-c",
    ...               "ls -l non_existent_file ; exit 0"],
    ...              stderr=STDOUT)
    'ls: non_existent_file: No such file or directory\n'
    """
    from subprocess import Popen, PIPE
    if 'stdout' in kwargs:
        raise ValueError('stdout argument not allowed, it will be overridden.')
    process = Popen(stdout=PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        raise CalledProcessError(retcode, cmd, output=output)
    return output







def run_as_subprocess(module_name, function_name, *args, **kwargs):
    """
    Run the named function as a subprocess.
    """
    import tempfile, subprocess, cPickle, os, logging
    _args_handle, args_filename = tempfile.mkstemp()
    _result_handle, result_filename = tempfile.mkstemp()
    try:
        to_pickle = args, kwargs
        cPickle.dump(to_pickle, open(args_filename, 'w'))
        python_code = "from %s import %s as fn; from cookbook.function_as_task import do_task; do_task(fn, '%s', '%s')" % (
            module_name, function_name, args_filename, result_filename
        )
        args = ('python%d.%d' % sys.version_info[:2], '-c', python_code)
        logging.debug('Running: %s', ' '.join(args))
        check_output(args=args, stderr=subprocess.STDOUT)
        #subprocess.check_call(args=args)
        result = cPickle.load(open(result_filename))
    finally:
        os.remove(args_filename) # no longer needed
        os.remove(result_filename) # no longer needed
    return result



def do_task(fn, args_filename, result_filename):
    "Actually do the task."
    import cPickle
    args, kw_args = cPickle.load(open(args_filename))
    result = fn(*args, **kw_args)
    cPickle.dump(result, open(result_filename, 'w'))





def sleep(time_to_sleep):
    "Function to test run function as task functionality."
    import time, logging
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")
    logging.info('Sleeping for %d seconds', time_to_sleep)
    time.sleep(time_to_sleep)
    logging.info('Finished sleeping for %d seconds', time_to_sleep)
    return "Slept for %d seconds" % time_to_sleep




def log_exception(exc_type, exc_obj, exc_tb):
    "Log an exception. Can be used as replacement for sys.excepthook."
    import traceback, logging
    exc_info = traceback.format_exception(exc_type, exc_obj, exc_tb)
    for l in exc_info:
        logging.error(l.strip())





def create_worker_on_queue(q, do_work):
    "Create a worker function that takes tasks off a queue."
    import sys, logging
    def worker():
        while True:
            item = q.get()
            try:
                result = do_work(item)
                logging.debug('Got result: %s', result)
            except:
                logging.error('Caught exception in queue worker')
                log_exception(*sys.exc_info())
            q.task_done()
    return worker




def create_queue(num_worker_threads, do_work):
    "Create a queue and a number of worker threads on it."
    from Queue import Queue
    from threading import Thread
    q = Queue()
    for _ in range(num_worker_threads):
        t = Thread(target=create_worker_on_queue(q, do_work))
        t.setDaemon(True)
        t.start()
    return q



if '__main__' == __name__:
    import logging
    logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")

    # the function that does our work
    def do_work(time_to_sleep):
        return run_as_subprocess("cookbook.function_as_task", "sleep", time_to_sleep)

    # create a queue
    num_worker_threads = 2
    q = create_queue(num_worker_threads, do_work)

    # put some work on it
    for item in range(5):
        q.put(item)

    # block until all tasks are done
    q.join()



#
# Copyright John Reid 2008, 2010
#

"""
Some utilities to use with matplotlib.
"""

import pylab as P, logging, math
from functools import wraps

_logger = logging.getLogger(__name__)

simple_colours = [
    'b',
    'g',
    'r',
    'c',
    'm',
    'y',
    'k',
]
"Basic pylab colours."



def pylab_ioff(fn):
    """
    Decorator that turns pylab interactive mode off for duration of call to decorated function.
    """
    @wraps(fn)
    def wrapper(*args, **kwds):
        interactive_mode = P.isinteractive()
        if interactive_mode:
            #logging.debug('Turning pylab interactive off.')
            P.ioff()
        try:
            return fn(*args, **kwds)
        finally:
            if interactive_mode:
                #logging.debug('Turning pylab interactive on.')
                P.ion()
    
    return wrapper


def set_rcParams_for_latex(fig_width_pt=None):
    """
    Set the rcParams in matplotlib to use postscript output.
    
    If fig_width_pt is given then set default figure size to match.
    """
    params = {
        'backend'         : 'ps',
        'text.usetex'     : True,
    }
    P.rcParams.update(params)
    
    if None != fig_width_pt:
        P.rcParams['figure.figsize'] = get_fig_size_for_latex(fig_width_pt)


def get_fig_size_for_latex(fig_width_pt=345.):
    """
    Get fig_width_pt from LaTeX using \showthe\columnwidth
    
    @return: figure size tuple if fig_width_pt was specified
    """
    inches_per_pt = 1.0/72.27                 # Convert pt to inch
    golden_mean = (math.sqrt(5) - 1.0) / 2.0  # Aesthetic ratio
    fig_width = fig_width_pt * inches_per_pt  # width in inches
    fig_height = fig_width * golden_mean      # height in inches
    return [fig_width, fig_height]



def _choose_other_dim(num_sub_plots, dim):
    return (num_sub_plots - 1) / dim + 1


def layout_sub_plot(num_sub_plots, num_rows=0, num_cols=0):
    "Specify either num_rows or num_cols."
    
    # make sure are positive
    if num_rows < 0:
        raise ValueError('num_rows must be positive.')
    if num_cols < 0:
        raise ValueError('num_cols must be positive.')

    # are both specified?
    if num_rows and num_cols:
        if (num_rows-1) * num_cols >= num_sub_plots:
            raise ValueError('Specified both num_rows and num_cols but could have used fewer num_rows.')
        if (num_cols-1) * num_rows >= num_sub_plots:
            raise ValueError('Specified both num_rows and num_cols but could have used fewer num_cols.')
    
    # is num_rows specified?
    elif num_rows:
        num_cols = _choose_other_dim(num_sub_plots, num_rows)

    # is num_cols specified?
    elif num_cols:
        num_rows = _choose_other_dim(num_sub_plots, num_cols)
        
    # make as square as possible
    else:
        num_rows = int(math.sqrt(num_sub_plots))
        num_cols = _choose_other_dim(num_sub_plots, num_rows)
    
    return num_rows, num_cols
        



def violin_plot(ax, data, pos, bp=False, facecolor='y', alpha=0.3):
    '''
    create violin plots on an axis
    '''
    from scipy.stats import gaussian_kde
    from numpy import linspace
    dist = max(pos) - min(pos)
    w = min(0.15 * max(dist, 1.0), 0.5)
    kdes = map(gaussian_kde, data)
    mins = [k.dataset.min() for k in kdes]
    maxs = [k.dataset.max() for k in kdes]
    xs = [linspace(m, M) for m, M in zip(mins, maxs)]
    vs = [k.evaluate(x) for k, x in zip(kdes, xs)]
    scaling = w / max(v.max() for v in vs)
    vs = [v * scaling for v in vs]
    for x, p, v in zip(xs, pos, vs):
        ax.fill_betweenx(x, -v+p,  v+p, facecolor=facecolor, alpha=alpha)
    if bp:
        ax.boxplot(data, notch=1, positions=pos, vert=1)




if '__main__' == __name__:
    
    class TestPylabIOff(object):
        @pylab_ioff
        def some_method(self, title, data):
            logging.debug('Plotting')
            P.figure()
            P.plot(data)
            P.title(title)
            P.savefig('test_figure.png')
            P.show()
            P.close()


    @pylab_ioff
    def some_normal_function(title, data):
        logging.debug('Plotting')
        P.figure()
        P.plot(data)
        P.title(title)
        P.savefig('test_other_figure.png')
        P.close()

    logging.basicConfig(level=logging.DEBUG)
    _logger.setLevel(logging.DEBUG)
    
    P.ion()

    test = TestPylabIOff()
    logging.debug('Trying some_method()')
    test.some_method('Title', [0,1,2,3,4])
    logging.debug('Trying some_normal_function()')
    some_normal_function('Title', [0,1,2,3,4])



#
# Copyright John Reid 2010
#

"""
Code to ease using pytables.
"""


import tables as T
from functools import wraps



def add_to_table(table):
    """
    @return: A decorator that adds arguments and the result of the function to the table.
    """
    def decorator(f):
        """
        A decorator that adds the arguments and result of its argument to a table.
        """
        @wraps(f)
        def wrapper(*args, **kwds):
            "Wrapper function that adds a row of the arguments and the result to a table."
            if kwds:
                raise ValueError('add_to_table decorator cannot handle keyword arguments')
            result = f(*args, **kwds)
            table.append([args + result])
            return result
        return wrapper
    return decorator


def _format_value(table, arg_name, value):
    "@return: The value as a string which may be quoted if necessary."
    if 'string' == table.coltypes[arg_name]:
        return '"%s"' % value
    else:
        return str(value)


def cache_in_table(table, *arg_names):
    """
    @return: A decorator that caches the arguments and the result of the function in the table.
    """
    def decorator(f):
        """
        A decorator that caches the arguments and result of its argument to a table.
        """
        # check arguments are not floats
        for name, t in table.coltypes.iteritems():
            if t.startswith('float'):
                raise ValueError('cache_in_table does not support matching floating point arguments: %s.' % name)
        
        # decorate with add_to_table
        f = add_to_table(table)(f)
        
        @wraps(f)
        def wrapper(*args, **kwds):
            """
            Wrapper function that inspects the table for a matching row and retrieves result or 
            calls function if there is no matching row.
            """
            # check arguments
            if kwds:
                raise ValueError('add_to_table decorator cannot handle keyword arguments')
            if len(args) != len(arg_names):
                raise ValueError('Expecting same number of arguments as names.')
            
            # build a condition to test how many rows we have that match the arguments
            condition = ' & '.join(
                '(%s == %s)' % (arg_name, _format_value(table, arg_name, str(value)))
                for arg_name, value in zip(arg_names, args)
            )
            
            # find the results that match the arguments
            matches = table.getWhereList(condition, condvars={})

            # Either return the existing row in the table or call the function and add a new one.            
            if len(matches) > 1:
                raise ValueError('Got more than one match in table.')
            elif 1 == len(matches):
                # get result from table
                row = table[matches[0]]
            else:
                # call function to get result
                row = f(*args)
            return tuple(row)[len(args):]
        
        return wrapper
    
    return decorator



if '__main__' == __name__:
    
    class TestRow(T.IsDescription):
        str_var     = T.StringCol(16, pos=1)
        int_var     = T.Int64Col(pos=2)
        float_var   = T.Float32Col(pos=3)
        result      = T.Float32Col(pos=4)
    
    #
    # Test the add_to_table decorator by putting some rows in a table
    #
    try:
        h5file
    except NameError:
        h5file = T.openFile("pytables_utils.h5", mode = "a", title = "Test file")
        
    try:
        test_table = h5file.root.Test.test
    except T.NoSuchNodeError:
        test_group = h5file.createGroup("/", 'Test', 'Test group')
        test_table = h5file.createTable(test_group, 'test', TestRow, "Test table")
    
    @add_to_table(test_table)
    def test_add_fn(str_var, int_var, float_var):
        return (len(str_var) * int_var + float_var,)
    
    @cache_in_table(test_table, 'str_var', 'int_var', 'float_var')
    def test_cache_fn(str_var, int_var, float_var):
        return (len(str_var) * int_var + float_var,)
    
#    test_add_fn('AA'  , 1, .5)
#    test_add_fn('AAAA', 2, .6)
#    test_add_fn('CG'  , 3, .7)
    
    test_cache_fn('aa'  , 1, .5)
    test_cache_fn('AAAA', 2, .5)
    test_cache_fn('CG'  , 3, .5)
    
    #
    # Inspect the table to e
    #
    for row in test_table:
        print row

    h5file.close()
    del h5file
    
    



#
# Copyright John Reid 2009
#

"""
Code to handle permutations and their inverses.
"""


def identity_permutation(size):
    "@return: The identity permutation."
    return range(size)


def invert_permutation(permutation):
    "@return: The inversion of the permutation."
    result = [None] * len(permutation)
    for i, x in enumerate(permutation):
        result[x] = i
    return result


def chain_permutations(permutation1, permutation2):
    "@return: A permutation that consists of the 2 permutations chained together."
    return [permutation2[p] for p in permutation1]


def permute(x, permutation):
    "@return: A list that is the permuted version of x."
    return list(permute_yield(x, permutation))


def permute_yield(x, permutation):
    "A generator that yields a permuted version of x."
    for p in permutation:
        yield x[p]




if '__main__' == __name__:
    import numpy.random
    size = 10
    permutation = numpy.random.permutation(size)

    def assert_sequences_equal(s1, s2):
        "Assert 2 sequences are identical."
        assert len(s1) == len(s2)
        for x1, x2 in zip(s1, s2):
            assert x1 == x2

    # check chaining inversion results in identity permutation
    assert_sequences_equal(identity_permutation(size), chain_permutations(invert_permutation(permutation), permutation))
    assert_sequences_equal(identity_permutation(size), chain_permutations(permutation, invert_permutation(permutation)))

    # check double inversion is same as permutation
    inversion = invert_permutation(permutation)
    double_inversion = invert_permutation(inversion)
    assert_sequences_equal(permutation, double_inversion)

    # check data permuted correctly
    x = numpy.random.randint(0, 100, size=size)
    permuted = permute(x, permutation)
    for i, p in enumerate(permutation):
        assert x[p] == permuted[i]

    # check inversion of permuted data is correct
    assert_sequences_equal(permute(permuted, inversion), x)



# 
# Taken from http://code.activestate.com/recipes/500261-named-tuples/
# Released under PSF license.
#

"""
From http://code.activestate.com/recipes/500261-named-tuples/
"""

#
# Try to import from collections module
#
try:
    from collections import namedtuple

except ImportError:

    #
    # If cannot import then spin a home grown version from python cookbook
    # http://code.activestate.com/recipes/500261-named-tuples/
    #
    from operator import itemgetter as _itemgetter
    from keyword import iskeyword as _iskeyword
    import sys as _sys
    
    def namedtuple(typename, field_names, verbose=False):
        """Returns a new subclass of tuple with named fields.
    
        >>> Point = namedtuple('Point', 'x y')
        >>> Point.__doc__                   # docstring for the new class
        'Point(x, y)'
        >>> p = Point(11, y=22)             # instantiate with positional args or keywords
        >>> p[0] + p[1]                     # indexable like a plain tuple
        33
        >>> x, y = p                        # unpack like a regular tuple
        >>> x, y
        (11, 22)
        >>> p.x + p.y                       # fields also accessable by name
        33
        >>> d = p._asdict()                 # convert to a dictionary
        >>> d['x']
        11
        >>> Point(**d)                      # convert from a dictionary
        Point(x=11, y=22)
        >>> p._replace(x=100)               # _replace() is like str.replace() but targets named fields
        Point(x=100, y=22)
    
        """
    
        # Parse and validate the field names.  Validation serves two purposes,
        # generating informative error messages and preventing template injection attacks.
        if isinstance(field_names, basestring):
            field_names = field_names.replace(',', ' ').split() # names separated by whitespace and/or commas
        field_names = tuple(field_names)
        for name in (typename,) + field_names:
            if not min(c.isalnum() or c=='_' for c in name):
                raise ValueError('Type names and field names can only contain alphanumeric characters and underscores: %r' % name)
            if _iskeyword(name):
                raise ValueError('Type names and field names cannot be a keyword: %r' % name)
            if name[0].isdigit():
                raise ValueError('Type names and field names cannot start with a number: %r' % name)
        seen_names = set()
        for name in field_names:
            if name.startswith('_'):
                raise ValueError('Field names cannot start with an underscore: %r' % name)
            if name in seen_names:
                raise ValueError('Encountered duplicate field name: %r' % name)
            seen_names.add(name)
    
        # Create and fill-in the class template
        numfields = len(field_names)
        argtxt = repr(field_names).replace("'", "")[1:-1]   # tuple repr without parens or quotes
        reprtxt = ', '.join('%s=%%r' % name for name in field_names)
        dicttxt = ', '.join('%r: t[%d]' % (name, pos) for pos, name in enumerate(field_names))
        template = '''class %(typename)s(tuple):
            '%(typename)s(%(argtxt)s)' \n
            __slots__ = () \n
            _fields = %(field_names)r \n
            def __new__(cls, %(argtxt)s):
                return tuple.__new__(cls, (%(argtxt)s)) \n
            @classmethod
            def _make(cls, iterable, new=tuple.__new__, len=len):
                'Make a new %(typename)s object from a sequence or iterable'
                result = new(cls, iterable)
                if len(result) != %(numfields)d:
                    raise TypeError('Expected %(numfields)d arguments, got %%d' %% len(result))
                return result \n
            def __repr__(self):
                return '%(typename)s(%(reprtxt)s)' %% self \n
            def _asdict(t):
                'Return a new dict which maps field names to their values'
                return {%(dicttxt)s} \n
            def _replace(self, **kwds):
                'Return a new %(typename)s object replacing specified fields with new values'
                result = self._make(map(kwds.pop, %(field_names)r, self))
                if kwds:
                    raise ValueError('Got unexpected field names: %%r' %% kwds.keys())
                return result \n\n''' % locals()
        for i, name in enumerate(field_names):
            template += '        %s = property(itemgetter(%d))\n' % (name, i)
        if verbose:
            print template
    
        # Execute the template string in a temporary namespace
        namespace = dict(itemgetter=_itemgetter)
        try:
            exec template in namespace
        except SyntaxError, e:
            raise SyntaxError(e.message + ':\n' + template)
        result = namespace[typename]
    
        # For pickling to work, the __module__ variable needs to be set to the frame
        # where the named tuple is created.  Bypass this step in enviroments where
        # sys._getframe is not defined (Jython for example).
        if hasattr(_sys, '_getframe'):
            result.__module__ = _sys._getframe(1).f_globals['__name__']
    
        return result
    
NamedTuple = namedtuple
    
    
    
    
    
if __name__ == '__main__':
    # verify that instances can be pickled
    from cPickle import loads, dumps
    Point = namedtuple('Point', 'x, y', True)
    p = Point(x=10, y=20)
    assert p == loads(dumps(p))

    # test and demonstrate ability to override methods
    class Point(namedtuple('Point', 'x y')):
        @property
        def hypot(self):
            return (self.x ** 2 + self.y ** 2) ** 0.5
        def __str__(self):
            return 'Point: x=%6.3f y=%6.3f hypot=%6.3f' % (self.x, self.y, self.hypot)

    for p in Point(3,4), Point(14,5), Point(9./7,6):
        print p

    class Point(namedtuple('Point', 'x y')):
        'Point class with optimized _make() and _replace() without error-checking'
        _make = classmethod(tuple.__new__)
        def _replace(self, _map=map, **kwds):
            return self._make(_map(kwds.get, ('x', 'y'), self))

    print Point(11, 22)._replace(x=100)

    import doctest
    TestResults = namedtuple('TestResults', 'failed attempted')
    print TestResults(*doctest.testmod())



#
# Copyright John Reid 2009
#



"""
A red-black tree implementation.
"""



class rbnode(object):
    """
    A node in a red black tree. See Cormen, Leiserson, Rivest, Stein 2nd edition pg 273.
    """

    def __init__(self, key):
        "Construct."
        self._key = key
        self._red = False
        self._left = None
        self._right = None
        self._p = None

    key = property(fget=lambda self: self._key, doc="The node's key")
    red = property(fget=lambda self: self._red, doc="Is the node red?")
    left = property(fget=lambda self: self._left, doc="The node's left child")
    right = property(fget=lambda self: self._right, doc="The node's right child")
    p = property(fget=lambda self: self._p, doc="The node's parent")

    def __str__(self):
        "String representation."
        return str(self.key)


    def __repr__(self):
        "String representation."
        return str(self.key)









class rbtree(object):
    """
    A red black tree. See Cormen, Leiserson, Rivest, Stein 2nd edition pg 273.
    """


    def __init__(self, create_node=rbnode):
        "Construct."

        self._nil = create_node(key=None)
        "Our nil node, used for all leaves."

        self._root = self.nil
        "The root of the tree."

        self._create_node = create_node
        "A callable that creates a node."


    root = property(fget=lambda self: self._root, doc="The tree's root node")
    nil = property(fget=lambda self: self._nil, doc="The tree's nil node")


    def search(self, key, x=None):
        """
        Search the subtree rooted at x (or the root if not given) iteratively for the key.

        @return: self.nil if it cannot find it.
        """
        if None == x:
            x = self.root
        while x != self.nil and key != x.key:
            if key < x.key:
                x = x.left
            else:
                x = x.right
        return x


    def minimum(self, x=None):
        """
        @return: The minimum value in the subtree rooted at x.
        """
        if None == x:
            x = self.root
        while x.left != self.nil:
            x = x.left
        return x


    def maximum(self, x=None):
        """
        @return: The maximum value in the subtree rooted at x.
        """
        if None == x:
            x = self.root
        while x.right != self.nil:
            x = x.right
        return x


    def insert_key(self, key):
        "Insert the key into the tree."
        self.insert_node(self._create_node(key=key))


    def insert_node(self, z):
        "Insert node z into the tree."
        y = self.nil
        x = self.root
        while x != self.nil:
            y = x
            if z.key < x.key:
                x = x.left
            else:
                x = x.right
        z._p = y
        if y == self.nil:
            self._root = z
        elif z.key < y.key:
            y._left = z
        else:
            y._right = z
        z._left = self.nil
        z._right = self.nil
        z._red = True
        self._insert_fixup(z)


    def _insert_fixup(self, z):
        "Restore red-black properties after insert."
        while z.p.red:
            if z.p == z.p.p.left:
                y = z.p.p.right
                if y.red:
                    z.p._red = False
                    y._red = False
                    z.p.p._red = True
                    z = z.p.p
                else:
                    if z == z.p.right:
                        z = z.p
                        self._left_rotate(z)
                    z.p._red = False
                    z.p.p._red = True
                    self._right_rotate(z.p.p)
            else:
                y = z.p.p.left
                if y.red:
                    z.p._red = False
                    y._red = False
                    z.p.p._red = True
                    z = z.p.p
                else:
                    if z == z.p.left:
                        z = z.p
                        self._right_rotate(z)
                    z.p._red = False
                    z.p.p._red = True
                    self._left_rotate(z.p.p)
        self.root._red = False


    def _left_rotate(self, x):
        "Left rotate x."
        y = x.right
        x._right = y.left
        if y.left != self.nil:
            y.left._p = x
        y._p = x.p
        if x.p == self.nil:
            self._root = y
        elif x == x.p.left:
            x.p._left = y
        else:
            x.p._right = y
        y._left = x
        x._p = y


    def _right_rotate(self, y):
        "Left rotate y."
        x = y.left
        y._left = x.right
        if x.right != self.nil:
            x.right._p = y
        x._p = y.p
        if y.p == self.nil:
            self._root = x
        elif y == y.p.right:
            y.p._right = x
        else:
            y.p._left = x
        x._right = y
        y._p = x


    def check_invariants(self):
        "@return: True iff satisfies all criteria to be red-black tree."

        def is_red_black_node(node):
            "@return: num_black"
            # check has _left and _right or neither
            if (node.left and not node.right) or (node.right and not node.left):
                return 0, False

            # check leaves are black
            if not node.left and not node.right and node.red:
                return 0, False

            # if node is red, check children are black
            if node.red and node.left and node.right:
                if node.left.red or node.right.red:
                    return 0, False

            # descend tree and check black counts are balanced
            if node.left and node.right:

                # check children's parents are correct
                if self.nil != node.left and node != node.left.p:
                    return 0, False
                if self.nil != node.right and node != node.right.p:
                    return 0, False

                # check children are ok
                left_counts, left_ok = is_red_black_node(node.left)
                if not left_ok:
                    return 0, False
                right_counts, right_ok = is_red_black_node(node.right)
                if not right_ok:
                    return 0, False

                # check children's counts are ok
                if left_counts != right_counts:
                    return 0, False
                return left_counts, True
            else:
                return 0, True

        num_black, is_ok = is_red_black_node(self.root)
        return is_ok and not self.root._red




def write_tree_as_dot(t, f, show_nil=False):
    "Write the tree in the dot language format to f."
    def node_id(node):
        return 'N%d' % id(node)

    def node_color(node):
        if node.red:
            return "red"
        else:
            return "black"

    def visit_node(node):
        "Visit a node."
        print >> f, "  %s [label=\"%s\", color=\"%s\"];" % (node_id(node), node, node_color(node))
        if node.left:
            if node.left != t.nil or show_nil:
                visit_node(node.left)
                print >> f, "  %s -> %s ;" % (node_id(node), node_id(node.left))
        if node.right:
            if node.right != t.nil or show_nil:
                visit_node(node.right)
                print >> f, "  %s -> %s ;" % (node_id(node), node_id(node.right))

    print >> f, "// Created by rbtree.write_dot()"
    print >> f, "digraph red_black_tree {"
    visit_node(t.root)
    print >> f, "}"




def test_tree(t, keys):
    "Insert keys one by one checking invariants and membership as we go."
    assert t.check_invariants()
    for i, key in enumerate(keys):
        for key2 in keys[:i]:
            assert t.nil != t.search(key2)
        for key2 in keys[i:]:
            assert (t.nil == t.search(key2)) ^ (key2 in keys[:i])
        t.insert_key(key)
        assert t.check_invariants()


if '__main__' == __name__:
    import os, sys, numpy.random as R
    def write_tree(t, filename):
        "Write the tree as an SVG file."
        f = open('%s.dot' % filename, 'w')
        write_tree_as_dot(t, f)
        f.close()
        os.system('dot %s.dot -Tsvg -o %s.svg' % (filename, filename))

    # test the rbtree
    R.seed(2)
    size=50
    keys = R.randint(-50, 50, size=size)
    t = rbtree()
    test_tree(t, keys)
    write_tree(t, 'tree')



#
# Copyright John Reid 2009
#



"""
Code to handle intervals.
"""



class Interval(object):
    """
    Represents an interval.
    Defined as half-open interval [start,end), which includes the start position but not the end.
    Start and end do not have to be numeric types.
    """


    def __init__(self, start, end):
        "Construct, start must be <= end."
        if start > end:
            raise ValueError('Start (%s) must not be greater than end (%s)' % (start, end))
        self._start = start
        self._end = end


    start = property(fget=lambda self: self._start, doc="The interval's start")
    end = property(fget=lambda self: self._end, doc="The interval's end")


    def __str__(self):
        "As string."
        return '[%s,%s)' % (self.start, self.end)


    def __repr__(self):
        "String representation."
        return '[%s,%s)' % (self.start, self.end)


    def __cmp__(self, other):
        "Compare."
        if None == other:
            return 1
        start_cmp = cmp(self.start, other.start)
        if 0 != start_cmp:
            return start_cmp
        else:
            return cmp(self.end, other.end)


    def __hash__(self):
        "Hash."
        return hash(self.start) ^ hash(self.end)


    def intersection(self, other):
        "Intersection. @return: An empty intersection if there is none."
        if self > other:
            other, self = self, other
        if self.end <= other.start:
            return Interval(self.start, self.start)
        return Interval(other.start, self.end)


    def hull(self, other):
        "@return: Interval containing both self and other."
        if self > other:
            other, self = self, other
        return Interval(self.start, other.end)


    def overlap(self, other):
        "@return: True iff self intersects other."
        if self > other:
            other, self = self, other
        return self.end > other.start


    def __contains__(self, item):
        "@return: True iff item in self."
        return self.start <= item and item < self.end


    def zero_in(self):
        "@return: True iff 0 in self."
        return self.start <= 0 and 0 < self.end


    def subset(self, other):
        "@return: True iff self is subset of other."
        return self.start >= other.start and self.end <= other.end


    def proper_subset(self, other):
        "@return: True iff self is proper subset of other."
        return self.start > other.start and self.end < other.end


    def empty(self):
        "@return: True iff self is empty."
        return self.start == self.end


    def singleton(self):
        "@return: True iff self.end - self.start == 1."
        return self.end - self.start == 1


    def separation(self, other):
        "@return: The distance between self and other."
        if self.overlap(other):
            return 0
        if self > other:
            return self.start - other.end
        else:
            return self.end - other.start


    def as_slice(self):
        "@return: A slice object that the interval indexes."
        return slice(self.start, self.end)



"""
Provide pre-/postconditions as function decorators.

Example usage:

  >>> def in_ge20(inval):
  ...    assert inval >= 20, 'Input value < 20'
  ...
  >>> def out_lt30(retval, inval):
  ...    assert retval < 30, 'Return value >= 30'
  ...
  >>> @precondition(in_ge20)
  ... @postcondition(out_lt30)
  ... def inc(value):
  ...   return value + 1
  ...
  >>> inc(5)
  Traceback (most recent call last):
    ...
  AssertionError: Input value < 20
  >>> inc(29)
  Traceback (most recent call last):
    ...
  AssertionError: Return value >= 30
  >>> inc(20)
  21

You can define as many pre-/postconditions for a function as you
like. It is also possible to specify both types of conditions at once:

  >>> @conditions(in_ge20, out_lt30)
  ... def add1(value):
  ...   return value + 1
  ...
  >>> add1(5)
  Traceback (most recent call last):
    ...
  AssertionError: Input value < 20

An interesting feature is the ability to prevent the creation of
pre-/postconditions at function definition time. This makes it
possible to use conditions for debugging and then switch them off for
distribution.

  >>> debug = False
  >>> @precondition(in_ge20, debug)
  ... def dec(value):
  ...   return value - 1
  ...
  >>> dec(5)
  4
"""

__all__ = [ 'precondition', 'postcondition', 'conditions' ]

DEFAULT_ON = True

def precondition(precondition, use_conditions=DEFAULT_ON):
    return conditions(precondition, None, use_conditions)

def postcondition(postcondition, use_conditions=DEFAULT_ON):
    return conditions(None, postcondition, use_conditions)

class conditions(object):
    __slots__ = ('__precondition', '__postcondition')

    def __init__(self, pre, post, use_conditions=DEFAULT_ON):
        if not use_conditions:
            pre, post = None, None

        self.__precondition  = pre
        self.__postcondition = post

    def __call__(self, function):
        # combine recursive wrappers (@precondition + @postcondition == @conditions)
        pres  = set( (self.__precondition,) )
        posts = set( (self.__postcondition,) )

        # unwrap function, collect distinct pre-/post conditions
        while type(function) is FunctionWrapper:
            pres.add(function._pre)
            posts.add(function._post)
            function = function._func

        # filter out None conditions and build pairs of pre- and postconditions
        conditions = map(None, filter(None, pres), filter(None, posts))

        # add a wrapper for each pair (note that 'conditions' may be empty)
        for pre, post in conditions:
            function = FunctionWrapper(pre, post, function)

        return function

class FunctionWrapper(object):
    def __init__(self, precondition, postcondition, function):
        self._pre  = precondition
        self._post = postcondition
        self._func = function

    def __call__(self, *args, **kwargs):
        precondition  = self._pre
        postcondition = self._post

        if precondition:
            precondition(*args, **kwargs)
        result = self._func(*args, **kwargs)
        if postcondition:
            postcondition(result, *args, **kwargs)
        return result

def __test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    __test()



"""

Adapted from: http://code.activestate.com/recipes/325205-cache-decorator-in-python-24/
(released under PSF license).

The latest version of Python introduced a new language feature, function and method decorators
(PEP 318, http://www.python.org/peps/pep-0318.html). This recipe show a common callable transformation
that can benefit from the new syntax, often referred to as Memoization pattern.

Memoization is a useful pattern for improving the amortized performance of a computationally expensive
function or method. The new syntax allows making explicit that a callable is memoized.

A restriction of the recipe above is that the arguments of the function (and also the method receiver
object for methods) have to be hashable.

The decorator can be generalized by allowing different caching policies (e.g. a FIFO cache or a cache
implementing an LRU policy) apart from the implied "cache-forever" policy of a dict.
"""

import logging, cPickle, os

def cachedmethod(function):
    return Memoize(function)
cached_method = cachedmethod

class Memoize(object):
    def __init__(self,function):
        self._cache = {}
        self._callable = function
        self.__name__ = function.__name__

    def __call__(self, *args, **kwds):
        cache = self._cache
        key = self._getKey(*args,**kwds)
        try: return cache[key]
        except KeyError:
            cachedValue = cache[key] = self._callable(*args,**kwds)
            return cachedValue

    def _getKey(self,*args,**kwds):
        return kwds and (args, ImmutableDict(kwds)) or args


class pickled_cached_method(object):
    def __init__(self, file, dump_on_update=True):
        self._file = file
        self.dump_on_update = dump_on_update
    def __call__(self, function):
        return PickledMemoize(function, self._file, dump_on_update=self.dump_on_update)


class PickledMemoize(object):
    """
    Like Memoize but pickles its cache/memo for use in next python session
    """

    def __init__(self, function, pickle_file, dump_on_update=True):
        "Constructor. pickle_file can either be a string naming the file or a callable that returns the name of the file."
        self._callable = function
        self._file = pickle_file
        self._cache = None
        self.__name__ = function.__name__
        self.dump_on_update = dump_on_update
        self.dirty = False
    
    def _get_file(self):
        "@return: The file we pickle to and from."
        # if the filename is a string then return it
        if isinstance(self._file, str):
            return self._file
        else: # otherwise assume it is a callable we can use to retrieve the name
            return self._file()
    
    file = property(_get_file, 'The name of the file we pickle the cache to and from.')

    def load_cache(self):
        "Load the memo from the cache."
        logging.info('Loading pickled memo from %s', self.file)
        self._cache = cPickle.load(open(self.file, 'rb'))

    def dump_cache(self):
        "Dump the memo to the cache."
        if self.dirty:
            logging.info('Dumping pickled memo to %s', self.file)
            cPickle.dump(self._get_cache(), open(self.file, 'wb'), protocol=2)
            self.dirty = False

    def _get_cache(self):
        "@return: The cache dictionary, loading it from disk if necessary and possible."
        if None == self._cache:
            if os.access(self.file, os.R_OK):
                self.load_cache()
            else:
                logging.info('No readable pickled memo cached to disk at %s, initialising empty cache', self.file)
                self._cache = {}
        return self._cache

    def __call__(self, *args, **kwds):
        "Call the function and cache result."
        cache = self._get_cache()
        key = self._getKey(*args,**kwds)
        try:
            return cache[key]
        except KeyError:
            cachedValue = cache[key] = self._callable(*args, **kwds)
            self.dirty = True
            if self.dump_on_update:
                self.dump_cache()
            return cachedValue

    def _getKey(self, *args, **kwds):
        "@return: The key for these arguments that indexes the cache dictionary."
        return kwds and (args, ImmutableDict(kwds)) or args


class ImmutableDict(dict):
    '''A hashable dict.'''

    def __init__(self,*args,**kwds):
        dict.__init__(self,*args,**kwds)
    def __setitem__(self,key,value):
        raise NotImplementedError, "dict is immutable"
    def __delitem__(self,key):
        raise NotImplementedError, "dict is immutable"
    def clear(self):
        raise NotImplementedError, "dict is immutable"
    def setdefault(self,k,default=None):
        raise NotImplementedError, "dict is immutable"
    def popitem(self):
        raise NotImplementedError, "dict is immutable"
    def update(self,other):
        raise NotImplementedError, "dict is immutable"
    def __hash__(self):
        return hash(tuple(self.iteritems()))



class pickled_method(object):
    "Pickles the result of the method (ignoring arguments) and uses this if possible on the next call."
    def __init__(self, file, name):
        self._file = file
        self._name = name

    def __call__(self, function):
        return Picklize(function, self._file, self._name)



class Picklize(object):
    def __init__(self, function, file, name):
        self._callable = function
        self._file = file
        self._name = name

    def __call__(self, *args, **kwds):
        try:
            logging.info('Trying to load %s from %s', self._name, self._file)
            return cPickle.load(open(self._file, 'rb'))
        except:
            logging.info('Could not load %s, will recalculate', self._name)
            result = self._callable(*args,**kwds)
            logging.info('Dumping %s to %s', self._name, self._file)
            cPickle.dump(result, open(self._file, 'wb'), protocol=2)
            return result




if __name__ == '__main__':
    from math import sqrt,log,sin,cos

    class Example:
        def __init__(self,x,y):
            # self._x and self._y should not be changed after initialization
            self._x = x
            self._y = y

        @cachedmethod
        def computeSomething(self, alpha, beta):
            w = log(alpha) * sqrt(self._x * alpha + self._y * beta)
            z = log(beta) * sqrt(self._x * beta + self._y * alpha)
            return sin(z) / cos(w)



"""
http://code.activestate.com/recipes/259173-groupby/
"""


class groupby(dict):
    def __init__(self, seq, key=lambda x:x):
        for value in seq:
            k = key(value)
            self.setdefault(k, []).append(value)
    __iter__ = dict.iteritems



"""
"""

class _const( object ):
    """
    In Python, any variable can be re-bound at will -- and modules don't 
    let you define special methods such as an instance's __setattr__ 
    to stop attribute re-binding. Easy solution (in Python 2.1 and up): 
    use an instance as 'module'...
    
    Adapted from http://code.activestate.com/recipes/65207-constants-in-python/
    (released under PSF license).
    """
    class ConstError( TypeError ): pass
    def __setattr__( self, name, value ):
        if name in self.__dict__:
            raise self.ConstError, "Can't rebind const(%s)" % name
        self.__dict__[name] = value
    def __delattr__( self, name ):
        if name in self.__dict__:
            raise self.ConstError, "Can't unbind const(%s)" % name
        raise NameError, name

import sys
sys.modules[__name__] = _const()



#
# Copyright John Reid 2007,2008
#


class DictOf(dict):
    "A dictionary where missing values are created by an __init__ time supplied function."

    def __init__(self, missing_value_creator, take_key_as_arg=False):
        "@arg missing_value_creator: Function to create missing values in the dictionary"
        if not take_key_as_arg:
            self.missing_value_creator = lambda k: missing_value_creator()
        else:
            self.missing_value_creator = missing_value_creator

    def __missing__(self, k):
        "Called when there is no value for a given key, k."
        value = self.missing_value_creator(k)
        self[k] = value
        return value


def dict_of(missing_value_creator, take_key_as_arg=False):
    "@return: A function that creates DictOf using these args."
    def result():
        return DictOf(missing_value_creator, take_key_as_arg)
    return result


class DictOfLists(dict):
    """A dictionary where the values are lists. If an accessed key is missing,
    the value is initialised to an empty list"""
    def __missing__(self, k):
        result = self[k] = list()
        return result


class DictOfSets(dict):
    """A dictionary where the values are sets. If an accessed key is missing,
    the value is initialised to an empty set"""
    def __missing__(self, k):
        result = self[k] = set()
        return result


class DictOfDicts(dict):
    """A dictionary where the values are dictionarys. If an accessed key is missing,
    the value is initialised to an empty dict"""
    def __missing__(self, k):
        result = self[k] = dict()
        return result


class DictOfInts(dict):
    """A dictionary where the values are ints. If an accessed key is missing,
    the value is initialised to 0. Useful for counting"""
    def __missing__(self, k):
        self[k] = 0
        return 0


class ChoiceDict(dict):
    """
    Dictionary that assigns one value in a sequence of values to each key. Can be useful for assigning colours/line styles in
    plots for example.
    """

    def __init__(self, values, cycle=False):
        "Construct with given values, if cycle==True then cycle through the values, otherwise raise exception when they are exhausted."
        dict.__init__(self)
        import itertools
        if cycle:
            self.it = itertools.cycle(values)
            "Iterator over values."
        else:
            self.it = iter(values)
            "Iterator over values."

    def __missing__(self, key):
        "Called when key is missing."
        self[key] = self.it.next()
        return self[key]

class UniqueIds(dict):
    """
    Holds a unique integer id for each key that is accessed.
    """

    def __init__(self):
        "Construct."
        self._next_id = 0

    def __missing__(self, key):
        "Called when the key is missing. Creates a new id for the key."
        self[key] = self._next_id
        self._next_id += 1
        return self[key]


if '__main__' == __name__:
    key = 'key'

    lists = DictOfLists()
    lists[key].append(1)
    print 'Should be [1]:', lists[key]

    sets = DictOfSets()
    sets[key].add(1)
    sets[key].add(1)
    print 'Should be set([1]):', sets[key]

    counts = DictOfInts()
    counts[key] += 1
    print 'Should be 1:', counts[key]



"""
Get console terminal size if possible.

From: http://stackoverflow.com/questions/566746/how-to-get-console-window-width-in-python
"""

def getTerminalSize():
    """
    Get console terminal size if possible.
    """

    def ioctl_GWINSZ(fd):
        try:
            import fcntl, termios, struct, os
            cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ,
        '1234'))
        except:
            return None
        return cr
    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
    if not cr:
        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = ioctl_GWINSZ(fd)
            os.close(fd)
        except:
            pass
    if not cr:
        try:
            cr = (env['LINES'], env['COLUMNS'])
        except:
            cr = (25, 80)
    return int(cr[1]), int(cr[0])



#
# Copyright John Reid 2007
#

"""
Code for lazy cached initialisation.
"""
import cPickle

class Cache(object):
    """
    Caches the result of calling a function. The function is evaluated once on the
    first invocation of self() and the cached result is returned for every
    invocation thereafter.
    """

    def __init__(self, initialiser):
        self.initialiser = initialiser
        self.instance = None

    def __call__(self):
        if None == self.instance:
            self.instance = self.initialiser()
        return self.instance



class PersistedCache(Cache):
    """
    Caches the result of calling a function. The function is evaluated once on the
    first invocation of self() and the cached result is returned for every
    invocation thereafter. The persist method stores the cached result in the
    given file. This pickled object is reused in preference to function
    invocation the next time the object is loaded.
    """
    def __init__(self, initialiser, pickle_file):
        Cache.__init__(self, initialiser)
        self.pickle_file = pickle_file

    def __call__(self):
        if None == self.instance:
            try:
                print 'Unpickling: %s' % self.pickle_file
                self.instance = cPickle.load(open(self.pickle_file))
            except:
                print 'Unpickling failed: %s' % self.pickle_file
                self.instance =  Cache.__call__(self)
        return self.instance

    def persist(self):
        "Pickle object to disk"
        print 'Pickling: %s' % self.pickle_file
        cPickle.dump(self(), open(self.pickle_file, 'w'))

def persist_all_in(variables):
    'Persist all the PersistedCaches in the given set of variables. E.g. persist_all_in(vars().values())'
    for var in variables:
        if isinstance(var, PersistedCache):
            var.persist()



"""
One-line decorator call adds caching to functions with hashable arguments and no
keyword arguments. When the maximum size is reached, the least recently used
entry is discarded -- appropriate for long-running processes which cannot allow
caches to grow without bound. Includes built-in performance instrumentation.

Adapted from http://code.activestate.com/recipes/498245-lru-and-lfu-cache-decorators/
(released under PSF license).
"""


from collections import deque
import pickle

def lru_cache(maxsize = 0, cache_storage_file=None):
    '''
    Decorator applying a least-recently-used cache with the given maximum size.

    Arguments to the cached function must be hashable.
    Cache performance statistics stored in f.hits and f.misses.
    '''
    def decorating_function(f):
        cache = {}        # mapping of args to results
        if maxsize:
            queue = deque()   # order that keys have been accessed
            refcount = {}     # number of times each key is in the access queue
        else:
            queue = refcount = None # no maxsize

        # load from file if possible
        if cache_storage_file:
            try:
                cache, queue, refcount = pickle.load(open(cache_storage_file))
            except:
                print 'Could not load cache from "%s"' % cache_storage_file

        def dump():
            if None == cache_storage_file:
                raise RuntimeError('No cache storage file specified')
            pickle.dump((cache, queue, refcount), open(cache_storage_file, 'w'))

        def wrapper(*args):
            # localize variable access (ugly but fast)
            _cache=cache; _len=len; _refcount=refcount; _maxsize=maxsize
            if _maxsize:
                queue_append=queue.append; queue_popleft = queue.popleft

            # get cache entry or compute if not found
            try:
                result = _cache[args]
                wrapper.hits += 1
            except KeyError:
                result = _cache[args] = f(*args)
                wrapper.misses += 1

            # do we have a maxsize for this cache?
            if None != queue:
                # record that this key was recently accessed
                queue_append(args)
                _refcount[args] = _refcount.get(args, 0) + 1

                # Purge least recently accessed cache contents
                while _len(_cache) > _maxsize:
                    k = queue_popleft()
                    _refcount[k] -= 1
                    if not _refcount[k]:
                        del _cache[k]
                        del _refcount[k]

                # Periodically compact the queue by duplicate keys
                if _len(queue) > _maxsize * 4:
                    for i in [None] * _len(queue):
                        k = queue_popleft()
                        if _refcount[k] == 1:
                            queue_append(k)
                        else:
                            _refcount[k] -= 1
                    assert len(queue) == len(cache) == len(refcount) == sum(refcount.itervalues()), (queue, cache, refcount)

            return result
        wrapper.__doc__ = f.__doc__
        wrapper.__name__ = f.__name__
        wrapper.hits = wrapper.misses = 0
        wrapper.cache = cache
        wrapper.persist_cache = dump
        return wrapper
    return decorating_function


if __name__ == '__main__':

    @lru_cache(cache_storage_file='lru_cache_test.cache')
    def f(x, y):
        return 3*x+y

    domain = range(5)
    from random import choice
    for i in xrange(1000):
        r = f(choice(domain), choice(domain))

    print f.hits, f.misses



#
# http://code.activestate.com/recipes/415903-two-dict-classes-which-can-lookup-keys-by-value-an/
# Released under PSF license
#

"""
From http://code.activestate.com/recipes/473899-progress-meter/
"""

class ReverseDict(dict):
    """
    A dictionary which can lookup values by key, and keys by value.
    All values and keys must be hashable, and unique.
    """
    def __init__(self, *args, **kw):
        dict.__init__(self, *args, **kw)
        self.reverse = dict((reversed(list(i)) for i in self.items()))

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        self.reverse[value] = key


class LookupDict(dict):
    """
    A dictionary which can lookup values by key, and keys by value.
    The lookup method returns a list of keys with matching values
    to the value argument.
    """
    def __init__(self, *args, **kw):
        dict.__init__(self, *args, **kw)

    def lookup(self, value):
        return [item[0] for item in self.items() if item[1] == value]



#
# Copyright John Reid 2009
#


"""
Decorator helpers.
"""


def simple_decorator(decorator):
    """This decorator can be used to turn simple functions
    into well-behaved decorators, so long as the decorators
    are fairly simple. If a decorator expects a function and
    returns a function (no descriptors), and if it doesn't
    modify function attributes or docstring, then it is
    eligible to use this. Simply apply @simple_decorator to
    your decorator and it will automatically preserve the
    docstring and function attributes of functions to which
    it is applied."""
    def new_decorator(f):
        g = decorator(f)
        g.__name__ = f.__name__
        g.__doc__ = f.__doc__
        g.__dict__.update(f.__dict__)
        return g
    # Now a few lines needed to make simple_decorator itself
    # be a well-behaved decorator.
    new_decorator.__name__ = decorator.__name__
    new_decorator.__doc__ = decorator.__doc__
    new_decorator.__dict__.update(decorator.__dict__)
    return new_decorator


class DecorateInstanceMethod(object):
    """
    Decorator that allows other decorators to decorate instance methods.
    """

    def __init__(self, callable):
        "Constructor."
        import warnings
        warnings.warn('This class is deprecated and may not be safe when used with more than one instance of self.obj.')
        
        self._callable = callable
        self.obj = None

    def __call__(self, *args, **kwds):
        if None != self.obj:
            return self._callable(self.obj, *args, **kwds)
        else:
            return self._callable(*args, **kwds)

    def __get__(descr, inst, instCls=None):
        descr.obj = inst
        return descr



#
# Copyright John Reid 2010
#

"""
Creates Venn diagrams from up to 4 sets.
"""

import numpy as N, scipy.optimize as O, warnings


def mid_points(xs):
    "Calculate mid_points"
    return [(xs[i] + xs[i+1])/2 for i in xrange(len(xs)-1)]

def calc_grid_areas(xs, ys):
    "Calculate the areas given by the grid bounds."
    return N.array(
        [
            [
                (xs[i+1] - xs[i]) * (ys[j+1] - ys[j])
                for i in xrange(len(xs)-1)
            ]
            for j in xrange(len(ys)-1)
        ]
    )


def calc_separation(bounds):
    "@return: A separation value. The closer to 0 it is, the better the bounds will be separated."
    xs = N.sort(bounds[:,:,0].flatten())
    ys = N.sort(bounds[:,:,1].flatten())
    def separation(zs):
        return N.sqrt(sum((zs[i] - zs[i+1])**-2 for i in xrange(len(zs)-1)))
    return separation(xs) + separation(ys)


def calc_squareness(bounds):
    "@return: A square-ness value. The closer to 0 it is, the more square the bounds will be."
    def squareness(bound):
        width = bound[1,0] - bound[0,0]
        height = bound[1,1] - bound[0,1]
        return (width > height and width / height or height / width) - 1
    return N.sqrt((N.array(map(squareness, bounds))**2).sum())


def calc_areas(bounds):
    """
    Calculate the areas given by these bounds.
    """
    assert bounds.shape == (4, 2, 2)
    xs = N.sort(bounds[:,:,0].flatten())
    ys = N.sort(bounds[:,:,1].flatten())
    mid_xs = mid_points(xs)
    mid_ys = mid_points(ys)
    grid_areas = calc_grid_areas(xs, ys)
    areas = N.zeros((2, 2, 2, 2))
    for i, x in enumerate(mid_xs):
        for j, y in enumerate(mid_ys):
            # which sets is it in?
            idx = tuple(b[0,0] < x and x < b[1,0] and b[0,1] < y and y < b[1,1] for b in bounds)
            areas[idx] += grid_areas[i,j]
    areas[0,0,0,0] = 0. # ignore those points outside all sets
    return areas



def calc_error(areas, sizes):
    "Calculate the error associated with these areas."
    assert areas[0,0,0,0] == sizes[0,0,0,0] == 0
    # rescale the areas
    areas = areas * (sizes.sum() / areas.sum())
    return N.sqrt(((areas - sizes)**2).sum())



def fit_venn4(sets):
    """
    Fit a Venn diagram to the 4 sets.
    """
    if 4 != len(sets):
        raise ValueError("Only works with 4 sets.")
    shape = (2, 2, 2, 2)
    sizes = N.zeros(shape, dtype=int) # sizes of all intersections
    union = reduce(set.union, sets)
    print 'Union has %d entries' % len(union)
    for idx in N.ndindex(2, 2, 2, 2):
        intersection = union
        for dim, intersect in enumerate(idx):
            if intersect:
                intersection = intersection.intersection(sets[dim])
            else:
                intersection = intersection.difference(sets[dim])
        sizes[idx] = len(intersection)
    assert sizes[0, 0, 0, 0] == 0
    assert sizes[1, 1, 1, 1] == len(reduce(set.intersection, sets))
    return sizes


def initial_bounds():
    "Return a suitable set of initial bounds."
    return N.array(
        [
            [[0, 0], [4, 6]],
            [[2, 1], [6, 4]],
            [[1, 3], [7, 5]],
            [[3, 2], [5, 7]],
        ],
        dtype=float
    ) / 8.


def find_bounds(sizes):
    """
    Use an optimizer to find the bounds.
    """
    bounds = 100.*initial_bounds().flatten()
    norm_sizes = 100.*sizes/sizes.sum()
    def func(x):
        bounds = x.reshape(4, 2, 2)
        areas = calc_areas(bounds)
        error = calc_error(areas, norm_sizes)
        # add terms to keep edges of sets away from each other
        separation = calc_separation(bounds)
        squareness = calc_squareness(bounds)
        #print error, separation, squareness
        return error + separation / 10. + squareness / 50.
    #xopt, fopt, iter, funcalls, warnflag = O.fmin(func, bounds, full_output=True)
    #xopt, fopt, gopt, Bopt, func_calls, grad_calls, warnflag = O.fmin_bfgs(func, bounds, full_output=True)
    #xopt, fopt, func_calls, grad_calls, warnflag = O.fmin_cg(func, bounds, full_output=True)
    xopt, fopt, direct, iter, funcalls, warnflag = O.fmin_powell(func, bounds, full_output=True)
    if 1 == warnflag:
        warnings.warn('Maximum number of function evaluations made.')
    elif 2 == warnflag:
        warnings.warn('Maximum number of iterations reached.')
    return xopt.reshape(4, 2, 2)/10.


def create_svg(bounds, sizes=None, labels=None, scale=120, font_size=50):
    """
    Create a SVG figure from the bounds.
    """
    import svgfig as S
    fig = S.Fig()
    colors = ["purple", "blue", "green", "yellow"]
    min_x = bounds[:,:,0].min()
    min_y = bounds[:,:,1].min()
    max_x = bounds[:,:,0].max()
    max_y = bounds[:,:,1].max()
    for rect, color in zip(bounds, colors):
        fig.d.append(
            S.SVG(
                "rect",
                x=scale*rect[0,0], y=scale*rect[0,1], width=scale*(rect[1,0]-rect[0,0]), height=scale*(rect[1,1]-rect[0,1]),
                fill=color, fill_opacity=".2",
                rx=10, ry=10,
                stroke_width=2
            )
        )
    if None != labels:
        fig.d.append(S.Text(scale*bounds[0, 0, 0], scale*bounds[0, 0, 1], labels[0], font_size=font_size, text_anchor="start"))
        fig.d.append(S.Text(scale*bounds[1, 1, 0], scale*bounds[1, 0, 1], labels[1], font_size=font_size, text_anchor="end"))
        fig.d.append(S.Text(scale*bounds[2, 1, 0], scale*bounds[2, 1, 1]+font_size, labels[2], font_size=font_size, text_anchor="end"))
        fig.d.append(S.Text(scale*bounds[3, 0, 0], scale*bounds[3, 1, 1]+font_size, labels[3], font_size=font_size, text_anchor="start"))
    return S.canvas(
        fig.SVG(),
        height="800px", width="800px",
        viewBox="%d %d %d %d" % (scale*min_x-font_size, scale*min_y-font_size, scale*(max_x-min_x)+2*font_size, scale*(max_y-min_y)+2*font_size))


if '__main__' == __name__:
    import numpy.random as R
    R.seed(2)

    test_sets = {
        'random' : (
            (
                set(R.randint(0, 1000, 100)),
                set(R.randint(0, 1000, 200)),
                set(R.randint(0, 1000, 300)),
                set(R.randint(0, 1000, 400)),
            ),
            (
                'A - 100',
                'B - 200',
                'C - 300',
                'D - 400',
            )
        ),
        'football' : (
            (
                set('manchester united'),
                set('arsenal'),
                set('liverpool'),
                set('chelsea'),
            ),
            (
                'manchester united',
                'arsenal',
                'liverpool',
                'chelsea',
            )
        )
    }
    for name, (sets, labels) in test_sets.iteritems():
        sizes = fit_venn4(sets)
        bounds = find_bounds(sizes)
        areas = calc_areas(bounds)
        error = calc_error(areas, sizes)
        print 'Final error: %f' % error
        svg = create_svg(bounds, sizes=sizes, labels=labels)
        svg.save('venn-4-%s.svg' % name)



#
# Copyright John Reid 2009
#

"""
Code to set up logging and options in a workflow
"""

import cPickle, logging, os
from cache_decorator import cachedmethod, pickled_cached_method


cache_dir = None


def get_cache_dir():
    return cache_dir


def output_cached_method(name):
    "@return: Decorator that stores output of methods in cache directory."
    return pickled_cached_method(os.path.join(get_cache_dir(), '%s.pickle' % name))


def caching_decorator(cache_name):
    "@return: A decorator that caches the results of the function."
    #print 'Calling caching_decorator: %s' % cache_name
    def decorator(func):
        "Decorates by caching (pickling) the results of the function."
        @cachedmethod
        def wrapper():
            "Wrapper to cache results of a function."
            pickle_file = os.path.join(get_cache_dir(), '%s.pickle' % cache_name)
            try:
                results = cPickle.load(open(pickle_file, 'rb'))
                logging.info('Unpickled %s', pickle_file)
            except:
                logging.info('Could not unpickle %s; executing %s()', pickle_file, func.__name__)
                results = func()
                logging.info('Pickling %s', pickle_file)
                cPickle.dump(results, open(pickle_file, 'wb'), protocol=2)
            return results
        return wrapper
    return decorator


def logger_has_file_handler(logger):
    "@return: Does the logger have a file handler already?"
    for handler in logger.handlers:
        if isinstance(handler, logging.FileHandler):
            return True
    return False


def ensure_dir_exists(dir):
    "Makes a directory if it does not already exist."
    if not os.access(dir, os.X_OK):
        logging.info('Making directory: %s', dir)
        os.makedirs(dir)


def log_attributes(module):
    "Log the attributes in a module."
    for attr in dir(module):
        if not attr.startswith('_'):
            logging.info('%30s : %s' % (attr, str(getattr(module, attr))))


def initialise_workflow(options):
    """
    Initialise the workflow:

    - initialise basic logging
    - over-ride options with any defined in options.output_dir/options.py
    - set up logging to file in options.output_dir
    - log option values
    - set up a cache directory to store cached output
    """
    #
    # Initialise basic logging (if not already done so)
    #
    logging.basicConfig(level=logging.INFO)

    #
    # Over-ride options with any defined in the output directory
    #
    options_py_file = os.path.join(options.output_dir, 'options.py')
    if os.path.exists(options_py_file):
        logging.info('Overriding options from %s' % options_py_file)
        execfile(options_py_file)
    else:
        logging.info('No options file: "%s" - using default options.' % options_py_file)

    ensure_dir_exists(options.output_dir)

    #
    # Now we have an output directory set up logging to a file
    #
    log_file = os.path.join(options.output_dir, 'log.txt')
    if not logger_has_file_handler(logging.getLogger()):
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(logging.Formatter('%(asctime)s:%(levelname)s:%(message)s'))
        logging.getLogger().addHandler(file_handler)

    logging.info('Options are:')
    log_attributes(options)

    logging.info('Initialising cache directory.')
    global cache_dir
    cache_dir = os.path.join(options.output_dir, 'cache')
    "the directory to put cached results into"
    ensure_dir_exists(cache_dir)
    logging.info('cache_dir: %s', get_cache_dir())



#
# Copyright John Reid 2011, 2012
#

from setuptools import setup, find_packages
import os


def read(*fnames):
    """
    Utility function to read the README file.
    Used for the long_description.  It's nice, because now 1) we have a top level
    README file and 2) it's easier to type in the README file than to put a raw
    string in below ...
    """
    return open(os.path.join(os.path.dirname(__file__), *fnames)).read()



#
# Call setup
#
setup(
    name             = 'cookbook',
    description      = 'Some useful Python code, mainly from the Python cookbook',
    long_description = read('python', 'cookbook', 'README'),
    version          = read('python', 'cookbook', 'VERSION').strip().split('-')[0],
    author           = 'John Reid',
    author_email     = 'john.reid@mrc-bsu.cam.ac.uk',
    url              = "http://sysbio.mrc-bsu.cam.ac.uk/johns/Cookbook/docs/build/html/",
    classifiers      = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries",
    ],
    
    packages         = find_packages(where='python'),
    py_modules       = ['process_priority'],
    package_dir      = { '' : 'python' },
    package_data     = { 'cookbook': ['README', 'LICENSE', 'VERSION'] },
    install_requires = ['distribute'],
)




def set_process_priority(pid=None,priority=1):
    """ Set The Priority of a Windows Process.  Priority is a value between 0-5 where
        2 is normal priority.  Default sets the priority of the current
        python process but can take any valid process ID. """

    import win32api,win32process,win32con

    priorityclasses = [win32process.IDLE_PRIORITY_CLASS,
                       win32process.BELOW_NORMAL_PRIORITY_CLASS,
                       win32process.NORMAL_PRIORITY_CLASS,
                       win32process.ABOVE_NORMAL_PRIORITY_CLASS,
                       win32process.HIGH_PRIORITY_CLASS,
                       win32process.REALTIME_PRIORITY_CLASS]
    if pid == None:
        pid = win32api.GetCurrentProcessId()
    handle = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, True, pid)
    win32process.SetPriorityClass(handle, priorityclasses[priority])

def make_current_process_nice():
    "Makes the current process have a lower priority"
    set_process_priority()



#
# Copyright John Reid 2009
#



"""
An augmented red-black tree that handles intervals.
"""


from rbtree import rbnode, rbtree, write_tree_as_dot, test_tree
from cookbook.interval import Interval
import logging


class rbintervalnode(rbnode):
    """
    A node of a red black tree of half-open intervals.
    """

    def __init__(self, key):
        "Construct."
        rbnode.__init__(self, key)
        self._max = self._key_max()

    max = property(fget=lambda self: self._max, doc="The maximum value of any endpoint in the subtree rooted at this node.")

    def _key_max(self):
        "@return: The maximum of the key of this node."
        if self.key:
            return self.key.end
        else:
            return None

    def _set_max_locally(self):
        "Use local information (i.e. children) to set max."
        self._max = max(self._key_max(), self.left.max, self.right.max)

    def __str__(self):
        "String representation."
        return '%s; max=%s' % (self.key, self.max)


    def __repr__(self):
        "String representation."
        return '%s; max=%s' % (self.key, self.max)

    def subtree_interval(self):
        "@return: An interval that the subtree rooted here spans."
        return Interval(self.key.start, self.max)





class rbintervaltree(rbtree):
    """
    A red black tree of half-open intervals. See Cormen, Leiserson, Rivest, Stein 2nd edition pg 311.
    """


    def __init__(self):
        "Construct."
        rbtree.__init__(self, create_node=rbintervalnode)


    def interval_search(self, i):
        "Search for a node whose interval overlaps i."
        x = self.root
        while x != self.nil and not i.overlap(x.key):
            if x.left != self.nil and x.left.max > i.start:
                x = x.left
            else:
                x = x.right
        return x


    def _propagate_augmented(self, z):
        "Propagate any changes to the augmented data up the tree."
        z._set_max_locally()
        while z.p != self.nil and z.p.max < z.max:
            z.p._max = max(z.p.max, z.max)
            z = z.p

    def _insert_fixup(self, z):
        "Restore max invariant properties after insert."
        self._propagate_augmented(z)
        rbtree._insert_fixup(self, z)

    def _left_rotate(self, x):
        "Left rotate and keep max values correct."
        rbtree._left_rotate(self, x)
        x.p._max = x.max
        x._set_max_locally()


    def _right_rotate(self, y):
        "Right rotate and keep max values correct."
        rbtree._right_rotate(self, y)
        y.p._max = y.max
        y._set_max_locally()


    def check_invariants(self):
        "Check the max invariant of the nodes and the red-black properties."
        def check_node(x):
            if self.nil == x:
                return x.max == None
            else:
                if x.max != max(x._key_max(), x.left.max, x.right.max):
                    assert False
                    return False
                return check_node(x.left) and check_node(x.right)

        return check_node(self.root) and rbtree.check_invariants(self)



    def find_closest_intervals(self, interval, max_distance=None):
        "@return: The closest intervals in the interval tree to the given interval."
        def visit_node(node, args):
            if None != node.key:
                closest_so_far = args[0]
                closest_distance_so_far = args[1]
                logging.debug(
                    'Looking for closest to %s; visiting %25s; best distance: %4s; closest so far: %s ',
                    interval, node, closest_distance_so_far, closest_so_far
                )

                distance = node.key.separation(interval)
                if abs(distance) < closest_distance_so_far:
                    args[0] = [node.key]
                    args[1] = abs(distance)
                elif abs(distance) == closest_distance_so_far:
                    args[0].append(node.key)

                # do we need to go any further?
                if node.key.start - interval.end <= closest_distance_so_far:
                    visit_node(node.right, args)

                if interval.start - node.max <= args[1]:
                    visit_node(node.left, args)

        if None == max_distance:
            max_distance = float('infinity')
        args = [[], max_distance]
        visit_node(self.root, args)

        return args



if '__main__' == __name__:
    logging.basicConfig(level=logging.DEBUG)
    def write_tree(t, filename):
        "Write the tree as an SVG file."
        f = open('%s.dot' % filename, 'w')
        write_tree_as_dot(t, f)
        f.close()
        os.system('dot %s.dot -Tsvg -o %s.svg' % (filename, filename))

    # test the rbtree
    import os, sys, numpy.random as R
    R.seed(2)
    size=50
    intervals = [Interval(start, start+length) for start, length in zip(R.randint(-5000, 5000, size=size), R.poisson(10., size=size))]
    interval_tree = rbintervaltree()
    test_tree(interval_tree, intervals)
    write_tree(interval_tree, 'interval_tree')
    print interval_tree.find_closest_intervals(Interval(3,30), max_distance=None)



#
# Copyright John Reid 2010
#

"""
Code to time things.
"""

import logging, time


class SimpleTimer(object):
    "Times things."
    
    def __init__(self):
        "Initialises and starts timer."
        self.restart()
    
    def restart(self):
        "Restarts the timer."
        self.start = time.time()
    
    def duration(self):
        "@return: The duration since start."
        return time.time() - self.start

    
class Timer(object):
    "Times things and implements the context management protocol (see PEP 343)."
    
    def __init__(self, msg=None, level=logging.INFO):
        self._msg = msg
        self._level = level
    
    def __enter__(self):
        self.timer = SimpleTimer()
        return self
        
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.duration = self.timer.duration()
        self._report_duration_(self.duration)
        return False
    
    def _report_duration_(self, duration):
        if self._msg:
            logging.log(self._level, 'Timer: Took %f seconds to %s', duration, self._msg)
        else:
            logging.log(self._level, 'Timer: Took %f seconds', duration)
            



#
# Taken from http://code.activestate.com/recipes/413486-first-class-enums-in-python/
# (released under PSF license)
#


"""
Taken from http://code.activestate.com/recipes/413486-first-class-enums-in-python/
(released under PSF license)
"""


def Enum(*names):
    ##assert names, "Empty enums are not supported" # <- Don't like empty enums? Uncomment!

    class EnumClass(object):
        __slots__ = names
        def __iter__(self):        return iter(constants)
        def __len__(self):         return len(constants)
        def __getitem__(self, i):  return constants[i]
        def __repr__(self):        return 'Enum' + str(names)
        def __str__(self):         return 'enum ' + str(constants)

    class EnumValue(object):
        __slots__ = ('__value')
        def __init__(self, value): self.__value = value
        Value = property(lambda self: self.__value)
        EnumType = property(lambda self: EnumType)
        def __hash__(self):        return hash(self.__value)
        def __cmp__(self, other):
            # C fans might want to remove the following assertion
            # to make all enums comparable by ordinal value {;))
            assert self.EnumType is other.EnumType, "Only values from the same enum are comparable"
            return cmp(self.__value, other.__value)
        def __invert__(self):      return constants[maximum - self.__value]
        def __nonzero__(self):     return bool(self.__value)
        def __repr__(self):        return str(names[self.__value])

    maximum = len(names) - 1
    constants = [None] * len(names)
    for i, each in enumerate(names):
        val = EnumValue(i)
        setattr(EnumClass, each, val)
        constants[i] = val
    constants = tuple(constants)
    EnumType = EnumClass()
    return EnumType


if __name__ == '__main__':
    print '\n*** Enum Demo ***'
    print '--- Days of week ---'
    Days = Enum('Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su')
    print Days
    print Days.Mo
    print Days.Fr
    print Days.Mo < Days.Fr
    print list(Days)
    for each in Days:
        print 'Day:', each
    print '--- Yes/No ---'
    Confirmation = Enum('No', 'Yes')
    answer = Confirmation.No
    print 'Your answer is not', ~answer
